##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2015 Benjamin Larsson <benjamin@southpole.se>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd

class SamplerateError(Exception):
    pass

class Decoder(srd.Decoder):
    api_version = 3
    id = 't55xx'
    name = 'T55xx'
    longname = 'RFID T55xx'
    desc = 'T55xx 100-150kHz RFID protocol.'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = []
    tags = ['IC', 'RFID']
    channels = (
        {'id': 'data', 'name': 'Data', 'desc': 'Data line'},
    )
    options = (
        {'id': 'coilfreq', 'desc': 'Coil frequency', 'default': 125000},
        {'id': 'start_gap', 'desc': 'Start gap min', 'default': 20},
        {'id': 'w_gap', 'desc': 'Write gap min', 'default': 20},
        {'id': 'w_one_min', 'desc': 'Write one min', 'default': 48},
        {'id': 'w_one_max', 'desc': 'Write one max', 'default': 63},
        {'id': 'w_zero_min', 'desc': 'Write zero min', 'default': 16},
        {'id': 'w_zero_max', 'desc': 'Write zero max', 'default': 31},
        {'id': 'em4100_decode', 'desc': 'EM4100 decode', 'default': 'on',
            'values': ('on', 'off')},
    )
    annotations = (
        ('bit_value', 'Bit value'),
        ('start_gap', 'Start gap'),
        ('write_gap', 'Write gap'),
        ('write_mode_exit', 'Write mode exit'),
        ('bit', 'Bit'),
        ('opcode', 'Opcode'),
        ('lock', 'Lock'),
        ('data', 'Data'),
        ('password', 'Password'),
        ('address', 'Address'),
        ('bitrate', 'Bitrate'),
    )
    annotation_rows = (
        ('bits', 'Bits', (0,)),
        ('structure', 'Structure', (1, 2, 3, 4)),
        ('fields', 'Fields', (5, 6, 7, 8, 9)),
        ('decode', 'Decode', (10,)),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.samplerate = None
        self.last_samplenum = None
        self.lastlast_samplenum = None
        self.state = 'START_GAP'
        self.bits_pos = [[0 for col in range(3)] for row in range(70)]
        self.br_string = ['RF/8', 'RF/16', 'RF/32', 'RF/40',
                          'RF/50', 'RF/64', 'RF/100', 'RF/128']
        self.mod_str1 = ['Direct', 'Manchester', 'Biphase', 'Reserved']
        self.mod_str2 = ['Direct', 'PSK1', 'PSK2', 'PSK3', 'FSK1', 'FSK2',
                         'FSK1a', 'FSK2a']
        self.pskcf_str = ['RF/2', 'RF/4', 'RF/8', 'Reserved']
        self.em4100_decode1_partial = 0

    def metadata(self, key, value):
        if key == srd.SRD_CONF_SAMPLERATE:
            self.samplerate = value
        self.field_clock = self.samplerate / self.options['coilfreq']
        self.wzmax = self.options['w_zero_max'] * self.field_clock
        self.wzmin = self.options['w_zero_min'] * self.field_clock
        self.womax = self.options['w_one_max'] * self.field_clock
        self.womin = self.options['w_one_min'] * self.field_clock
        self.startgap = self.options['start_gap'] * self.field_clock
        self.writegap = self.options['w_gap'] * self.field_clock
        self.nogap = 64 * self.field_clock

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def decode_config(self, idx):
        safer_key = self.bits_pos[idx][0]<<3 | self.bits_pos[idx+1][0]<<2 | \
                    self.bits_pos[idx+2][0]<<1 | self.bits_pos[idx+3][0]
        self.put(self.bits_pos[idx][1], self.bits_pos[idx+3][2], self.out_ann,
                 [10, ['Safer Key' + ': %X' % safer_key,'%X' % safer_key]])
        bitrate = self.bits_pos[idx+11][0]<<2 | self.bits_pos[idx+12][0]<<1 | \
                  self.bits_pos[idx+13][0]
        self.put(self.bits_pos[idx+11][1], self.bits_pos[idx+13][2],
                 self.out_ann, [10, ['Data Bit Rate: ' + \
                 self.br_string[bitrate], self.br_string[bitrate]]])
        modulation1 = self.bits_pos[idx+15][0]<<1 | self.bits_pos[idx+16][0]
        modulation2 = self.bits_pos[idx+17][0]<<2 | \
                      self.bits_pos[idx+18][0]<<1 | self.bits_pos[idx+19][0]
        if modulation1 == 0:
            mod_string = self.mod_str2[modulation2]
        else:
            mod_string = self.mod_str1[modulation1]
        self.put(self.bits_pos[idx+15][1], self.bits_pos[idx+19][2],
                 self.out_ann, [10, ['Modulation: ' + mod_string, mod_string]])
        psk_cf = self.bits_pos[idx+20][0]<<1 | self.bits_pos[idx+21][0]
        self.put(self.bits_pos[idx+20][1], self.bits_pos[idx+21][2],
                 self.out_ann, [10, ['PSK-CF: ' + self.pskcf_str[psk_cf],
                 self.pskcf_str[psk_cf]]])
        self.put(self.bits_pos[idx+22][1], self.bits_pos[idx+22][2],
                 self.out_ann, [10, ['AOR' + ': %d' % \
                 (self.bits_pos[idx+22][0]),'%d' % (self.bits_pos[idx+22][0])]])
        maxblock = self.bits_pos[idx+24][0]<<2 | self.bits_pos[idx+25][0]<<1 | \
                   self.bits_pos[idx+26][0]
        self.put(self.bits_pos[idx+24][1], self.bits_pos[idx+26][2],
                 self.out_ann, [10, ['Max-Block' + ': %d' % maxblock,
                 '%d' % maxblock]])
        self.put(self.bits_pos[idx+27][1], self.bits_pos[idx+27][2],
                 self.out_ann, [10, ['PWD' + ': %d' % \
                 (self.bits_pos[idx+27][0]),'%d' % (self.bits_pos[idx+27][0])]])
        self.put(self.bits_pos[idx+28][1], self.bits_pos[idx+28][2],
                 self.out_ann, [10, ['ST-sequence terminator' + ': %d' % \
                 (self.bits_pos[idx+28][0]),'%d' % (self.bits_pos[idx+28][0])]])
        self.put(self.bits_pos[idx+31][1], self.bits_pos[idx+31][2],
                 self.out_ann, [10, ['POR delay' + ': %d' % \
                 (self.bits_pos[idx+31][0]),'%d' % (self.bits_pos[idx+31][0])]])

    def put4bits(self, idx):
        bits = self.bits_pos[idx][0]<<3 | self.bits_pos[idx+1][0]<<2 | \
               self.bits_pos[idx+2][0]<<1 | self.bits_pos[idx+3][0]
        self.put(self.bits_pos[idx][1], self.bits_pos[idx+3][2], self.out_ann,
                 [10, ['%X' % bits]])

    def em4100_decode1(self, idx):
        self.put(self.bits_pos[idx][1], self.bits_pos[idx+8][2], self.out_ann,
                 [10, ['EM4100 header', 'EM header', 'Header', 'H']])
        self.put4bits(idx+9)
        self.put4bits(idx+14)
        self.put4bits(idx+19)
        self.put4bits(idx+24)
        self.em4100_decode1_partial = self.bits_pos[idx+29][0]<<3 | \
            self.bits_pos[idx+30][0]<<2 | self.bits_pos[idx+31][0]<<1
        self.put(self.bits_pos[idx+29][1], self.bits_pos[idx+31][2],
                 self.out_ann, [10, ['Partial nibble']])

    def em4100_decode2(self, idx):
        if self.em4100_decode1_partial != 0:
            bits = self.em4100_decode1_partial + self.bits_pos[idx][0]
            self.put(self.bits_pos[idx][1], self.bits_pos[idx][2],
                     self.out_ann, [10, ['%X' % bits]])
            self.em4100_decode1_partial = 0
        else:
            self.put(self.bits_pos[idx][1], self.bits_pos[idx][2],
                     self.out_ann, [10, ['Partial nibble']])

        self.put4bits(idx+2)
        self.put4bits(idx+7)
        self.put4bits(idx+12)
        self.put4bits(idx+17)
        self.put4bits(idx+22)
        self.put(self.bits_pos[idx+27][1], self.bits_pos[idx+31][2],
                 self.out_ann, [10, ['EM4100 trailer']])

    def get_32_bits(self, idx):
        retval = 0
        for i in range(0, 32):
            retval <<= 1
            retval |= self.bits_pos[i+idx][0]
        return retval

    def get_3_bits(self, idx):
        retval = self.bits_pos[idx][0]<<2 | self.bits_pos[idx+1][0]<<1 | \
                 self.bits_pos[idx+2][0]
        return retval

    def put_fields(self):
        if (self.bit_nr == 70):
            self.put(self.bits_pos[0][1], self.bits_pos[1][2], self.out_ann,
                     [5, ['Opcode' + ': %d%d' % (self.bits_pos[0][0],
                     self.bits_pos[1][0]), '%d%d' % (self.bits_pos[0][0],
                     self.bits_pos[1][0])]])
            password = self.get_32_bits(2)
            self.put(self.bits_pos[2][1], self.bits_pos[33][2], self.out_ann,
                     [8, ['Password' + ': %X' % password, '%X' % password]])
            self.put(self.bits_pos[34][1], self.bits_pos[34][2], self.out_ann,
                     [6, ['Lock' + ': %X' % self.bits_pos[34][0],
                     '%X' % self.bits_pos[34][0]]])
            data = self.get_32_bits(35)
            self.put(self.bits_pos[35][1], self.bits_pos[66][2], self.out_ann,
                     [7, ['Data' + ': %X' % data, '%X' % data]])
            addr = self.get_3_bits(67)
            self.put(self.bits_pos[67][1], self.bits_pos[69][2], self.out_ann,
                     [9, ['Addr' + ': %X' % addr, '%X' % addr]])
            if addr == 0:
                self.decode_config(35)
            if addr == 7:
                self.put(self.bits_pos[35][1], self.bits_pos[66][2],
                         self.out_ann, [10, ['Password' + ': %X' % data,
                         '%X' % data]])
            # If we are programming EM4100 data we can decode it halfway.
            if addr == 1 and self.options['em4100_decode'] == 'on':
                self.em4100_decode1(35)
            if addr == 2 and self.options['em4100_decode'] == 'on':
                self.em4100_decode2(35)

        if (self.bit_nr == 38):
            self.put(self.bits_pos[0][1], self.bits_pos[1][2], self.out_ann,
                     [5, ['Opcode' + ': %d%d' % (self.bits_pos[0][0],
                     self.bits_pos[1][0]), '%d%d' % (self.bits_pos[0][0],
                     self.bits_pos[1][0])]])
            self.put(self.bits_pos[2][1], self.bits_pos[2][2], self.out_ann,
                     [6, ['Lock' + ': %X' % self.bits_pos[2][0],
                     '%X' % self.bits_pos[2][0]]])
            data = self.get_32_bits(3)
            self.put(self.bits_pos[3][1], self.bits_pos[34][2], self.out_ann,
                     [7, ['Data' + ': %X' % data, '%X' % data]])
            addr = self.get_3_bits(35)
            self.put(self.bits_pos[35][1], self.bits_pos[37][2], self.out_ann,
                     [9, ['Addr' + ': %X' % addr, '%X' % addr]])
            if addr == 0:
                self.decode_config(3)
            if addr == 7:
                self.put(self.bits_pos[3][1], self.bits_pos[34][2],
                         self.out_ann, [10, ['Password' + ': %X' % data,
                         '%X' % data]])
            # If we are programming EM4100 data we can decode it halfway.
            if addr == 1 and self.options['em4100_decode'] == 'on':
                self.em4100_decode1(3)
            if addr == 2 and self.options['em4100_decode'] == 'on':
                self.em4100_decode2(3)

        if (self.bit_nr == 2):
            self.put(self.bits_pos[0][1], self.bits_pos[1][2], self.out_ann,
                     [5, ['Opcode' + ': %d%d' % (self.bits_pos[0][0],
                     self.bits_pos[1][0]), '%d%d' % (self.bits_pos[0][0],
                     self.bits_pos[1][0])]])
        self.bit_nr = 0

    def add_bits_pos(self, bit, bit_start, bit_end):
        if self.bit_nr < 70:
            self.bits_pos[self.bit_nr][0] = bit
            self.bits_pos[self.bit_nr][1] = bit_start
            self.bits_pos[self.bit_nr][2] = bit_end
            self.bit_nr += 1

    def decode(self):
        if not self.samplerate:
            raise SamplerateError('Cannot decode without samplerate.')

        self.last_samplenum = 0
        self.lastlast_samplenum = 0
        self.last_edge = 0
        self.oldpl = 0
        self.oldpp = 0
        self.oldsamplenum = 0
        self.last_bit_pos = 0
        self.old_gap_start = 0
        self.old_gap_end = 0
        self.gap_detected = 0
        self.bit_nr = 0

        while True:
            (pin,) = self.wait({0: 'e'})

            pl = self.samplenum - self.oldsamplenum
            pp = pin
            samples = self.samplenum - self.last_samplenum

            if self.state == 'WRITE_GAP':
                if pl > self.writegap:
                    self.gap_detected = 1
                    self.put(self.last_samplenum, self.samplenum,
                             self.out_ann, [2, ['Write gap']])
                if (self.last_samplenum-self.old_gap_end) > self.nogap:
                    self.gap_detected = 0
                    self.state = 'START_GAP'
                    self.put(self.old_gap_end, self.last_samplenum,
                             self.out_ann, [3, ['Write mode exit']])
                    self.put_fields()

            if self.state == 'START_GAP':
                if pl > self.startgap:
                    self.gap_detected = 1
                    self.put(self.last_samplenum, self.samplenum,
                             self.out_ann, [1, ['Start gap']])
                    self.state = 'WRITE_GAP'

            if self.gap_detected == 1:
                self.gap_detected = 0
                if (self.last_samplenum - self.old_gap_end) > self.wzmin \
                        and (self.last_samplenum - self.old_gap_end) < self.wzmax:
                    self.put(self.old_gap_end, self.last_samplenum,
                             self.out_ann, [0, ['0']])
                    self.put(self.old_gap_end, self.last_samplenum,
                             self.out_ann, [4, ['Bit']])
                    self.add_bits_pos(0, self.old_gap_end,
                                      self.last_samplenum)
                if (self.last_samplenum - self.old_gap_end) > self.womin \
                        and (self.last_samplenum - self.old_gap_end) < self.womax:
                    self.put(self.old_gap_end, self.last_samplenum,
                             self.out_ann, [0, ['1']])
                    self.put(self.old_gap_end, self.last_samplenum,
                             self.out_ann, [4, ['Bit']])
                    self.add_bits_pos(1, self.old_gap_end, self.last_samplenum)

                self.old_gap_start = self.last_samplenum
                self.old_gap_end = self.samplenum

            self.oldpl = pl
            self.oldpp = pp
            self.oldsamplenum = self.samplenum
            self.last_samplenum = self.samplenum



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2012-2014 Uwe Hermann <uwe@hermann-uwe.de>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd
from common.srdhelper import bcd2int

def reg_list():
    l = []
    for i in range(8 + 1):
        l.append(('reg-0x%02x' % i, 'Register 0x%02x' % i))

    return tuple(l)

class Decoder(srd.Decoder):
    api_version = 3
    id = 'rtc8564'
    name = 'RTC-8564'
    longname = 'Epson RTC-8564 JE/NB'
    desc = 'Realtime clock module protocol.'
    license = 'gplv2+'
    inputs = ['i2c']
    outputs = []
    tags = ['Clock/timing']
    annotations = reg_list() + (
        ('read', 'Read date/time'),
        ('write', 'Write date/time'),
        ('bit-reserved', 'Reserved bit'),
        ('bit-vl', 'VL bit'),
        ('bit-century', 'Century bit'),
        ('reg-read', 'Register read'),
        ('reg-write', 'Register write'),
    )
    annotation_rows = (
        ('bits', 'Bits', tuple(range(0, 8 + 1)) + (11, 12, 13)),
        ('regs', 'Register access', (14, 15)),
        ('date-time', 'Date/time', (9, 10)),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.state = 'IDLE'
        self.hours = -1
        self.minutes = -1
        self.seconds = -1
        self.days = -1
        self.weekdays = -1
        self.months = -1
        self.years = -1
        self.bits = []

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def putx(self, data):
        self.put(self.ss, self.es, self.out_ann, data)

    def putd(self, bit1, bit2, data):
        self.put(self.bits[bit1][1], self.bits[bit2][2], self.out_ann, data)

    def putr(self, bit):
        self.put(self.bits[bit][1], self.bits[bit][2], self.out_ann,
                 [11, ['Reserved bit', 'Reserved', 'Rsvd', 'R']])

    def handle_reg_0x00(self, b): # Control register 1
        pass

    def handle_reg_0x01(self, b): # Control register 2
        ti_tp = 1 if (b & (1 << 4)) else 0
        af = 1 if (b & (1 << 3)) else 0
        tf = 1 if (b & (1 << 2)) else 0
        aie = 1 if (b & (1 << 1)) else 0
        tie = 1 if (b & (1 << 0)) else 0

        ann = ''

        s = 'repeated' if ti_tp else 'single-shot'
        ann += 'TI/TP = %d: %s operation upon fixed-cycle timer interrupt '\
               'events\n' % (ti_tp, s)
        s = '' if af else 'no '
        ann += 'AF = %d: %salarm interrupt detected\n' % (af, s)
        s = '' if tf else 'no '
        ann += 'TF = %d: %sfixed-cycle timer interrupt detected\n' % (tf, s)
        s = 'enabled' if aie else 'prohibited'
        ann += 'AIE = %d: INT# pin output %s when an alarm interrupt '\
               'occurs\n' % (aie, s)
        s = 'enabled' if tie else 'prohibited'
        ann += 'TIE = %d: INT# pin output %s when a fixed-cycle interrupt '\
               'event occurs\n' % (tie, s)

        self.putx([1, [ann]])

    def handle_reg_0x02(self, b): # Seconds / Voltage-low bit
        vl = 1 if (b & (1 << 7)) else 0
        self.putd(7, 7, [12, ['Voltage low: %d' % vl, 'Volt. low: %d' % vl,
                        'VL: %d' % vl, 'VL']])
        s = self.seconds = bcd2int(b & 0x7f)
        self.putd(6, 0, [2, ['Second: %d' % s, 'Sec: %d' % s, 'S: %d' % s, 'S']])

    def handle_reg_0x03(self, b): # Minutes
        self.putr(7)
        m = self.minutes = bcd2int(b & 0x7f)
        self.putd(6, 0, [3, ['Minute: %d' % m, 'Min: %d' % m, 'M: %d' % m, 'M']])

    def handle_reg_0x04(self, b): # Hours
        self.putr(7)
        self.putr(6)
        h = self.hours = bcd2int(b & 0x3f)
        self.putd(5, 0, [4, ['Hour: %d' % h, 'H: %d' % h, 'H']])

    def handle_reg_0x05(self, b): # Days
        self.putr(7)
        self.putr(6)
        d = self.days = bcd2int(b & 0x3f)
        self.putd(5, 0, [5, ['Day: %d' % d, 'D: %d' % d, 'D']])

    def handle_reg_0x06(self, b): # Weekdays
        for i in (7, 6, 5, 4, 3):
            self.putr(i)
        w = self.weekdays = bcd2int(b & 0x07)
        self.putd(2, 0, [6, ['Weekday: %d' % w, 'WD: %d' % w, 'WD', 'W']])

    def handle_reg_0x07(self, b): # Months / century bit
        c = 1 if (b & (1 << 7)) else 0
        self.putd(7, 7, [13, ['Century bit: %d' % c, 'Century: %d' % c,
                              'Cent: %d' % c, 'C: %d' % c, 'C']])
        self.putr(6)
        self.putr(5)
        m = self.months = bcd2int(b & 0x1f)
        self.putd(4, 0, [7, ['Month: %d' % m, 'Mon: %d' % m, 'M: %d' % m, 'M']])

    def handle_reg_0x08(self, b): # Years
        y = self.years = bcd2int(b & 0xff)
        self.putx([8, ['Year: %d' % y, 'Y: %d' % y, 'Y']])

    def handle_reg_0x09(self, b): # Alarm, minute
        pass

    def handle_reg_0x0a(self, b): # Alarm, hour
        pass

    def handle_reg_0x0b(self, b): # Alarm, day
        pass

    def handle_reg_0x0c(self, b): # Alarm, weekday
        pass

    def handle_reg_0x0d(self, b): # CLKOUT output
        pass

    def handle_reg_0x0e(self, b): # Timer setting
        pass

    def handle_reg_0x0f(self, b): # Down counter for fixed-cycle timer
        pass

    def decode(self, ss, es, data):
        cmd, databyte = data

        # Collect the 'BITS' packet, then return. The next packet is
        # guaranteed to belong to these bits we just stored.
        if cmd == 'BITS':
            self.bits = databyte
            return

        # Store the start/end samples of this I²C packet.
        self.ss, self.es = ss, es

        # State machine.
        if self.state == 'IDLE':
            # Wait for an I²C START condition.
            if cmd != 'START':
                return
            self.state = 'GET SLAVE ADDR'
            self.ss_block = ss
        elif self.state == 'GET SLAVE ADDR':
            # Wait for an address write operation.
            # TODO: We should only handle packets to the RTC slave (0xa2/0xa3).
            if cmd != 'ADDRESS WRITE':
                return
            self.state = 'GET REG ADDR'
        elif self.state == 'GET REG ADDR':
            # Wait for a data write (master selects the slave register).
            if cmd != 'DATA WRITE':
                return
            self.reg = databyte
            self.state = 'WRITE RTC REGS'
        elif self.state == 'WRITE RTC REGS':
            # If we see a Repeated Start here, it's probably an RTC read.
            if cmd == 'START REPEAT':
                self.state = 'READ RTC REGS'
                return
            # Otherwise: Get data bytes until a STOP condition occurs.
            if cmd == 'DATA WRITE':
                r, s = self.reg, '%02X: %02X' % (self.reg, databyte)
                self.putx([15, ['Write register %s' % s, 'Write reg %s' % s,
                                'WR %s' % s, 'WR', 'W']])
                handle_reg = getattr(self, 'handle_reg_0x%02x' % self.reg)
                handle_reg(databyte)
                self.reg += 1
                # TODO: Check for NACK!
            elif cmd == 'STOP':
                # TODO: Handle read/write of only parts of these items.
                d = '%02d.%02d.%02d %02d:%02d:%02d' % (self.days, self.months,
                    self.years, self.hours, self.minutes, self.seconds)
                self.put(self.ss_block, es, self.out_ann,
                         [9, ['Write date/time: %s' % d, 'Write: %s' % d,
                              'W: %s' % d]])
                self.state = 'IDLE'
            else:
                pass # TODO
        elif self.state == 'READ RTC REGS':
            # Wait for an address read operation.
            # TODO: We should only handle packets to the RTC slave (0xa2/0xa3).
            if cmd == 'ADDRESS READ':
                self.state = 'READ RTC REGS2'
                return
            else:
                pass # TODO
        elif self.state == 'READ RTC REGS2':
            if cmd == 'DATA READ':
                r, s = self.reg, '%02X: %02X' % (self.reg, databyte)
                self.putx([15, ['Read register %s' % s, 'Read reg %s' % s,
                                'RR %s' % s, 'RR', 'R']])
                handle_reg = getattr(self, 'handle_reg_0x%02x' % self.reg)
                handle_reg(databyte)
                self.reg += 1
                # TODO: Check for NACK!
            elif cmd == 'STOP':
                d = '%02d.%02d.%02d %02d:%02d:%02d' % (self.days, self.months,
                    self.years, self.hours, self.minutes, self.seconds)
                self.put(self.ss_block, es, self.out_ann,
                         [10, ['Read date/time: %s' % d, 'Read: %s' % d,
                               'R: %s' % d]])
                self.state = 'IDLE'
            else:
                pass # TODO?



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2015 Stefan Brüns <stefan.bruens@rwth-aachen.de>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd
import struct

class SamplerateError(Exception):
    pass

class pcap_usb_pkt():
    # Linux usbmon format, see Documentation/usb/usbmon.txt
    h  = b'\x00\x00\x00\x00' # ID part 1
    h += b'\x00\x00\x00\x00' # ID part 2
    h += b'C'                # 'S'ubmit / 'C'omplete / 'E'rror
    h += b'\x03'             # ISO (0), Intr, Control, Bulk (3)
    h += b'\x00'             # Endpoint
    h += b'\x00'             # Device address
    h += b'\x00\x00'         # Bus number
    h += b'-'                # Setup tag - 0: Setup present, '-' otherwise
    h += b'<'                # Data tag - '<' no data, 0 otherwise
    # Timestamp
    h += b'\x00\x00\x00\x00' # TS seconds part 1
    h += b'\x00\x00\x00\x00' # TS seconds part 2
    h += b'\x00\x00\x00\x00' # TS useconds
    #
    h += b'\x00\x00\x00\x00' # Status 0: OK
    h += b'\x00\x00\x00\x00' # URB length
    h += b'\x00\x00\x00\x00' # Data length
    # Setup packet data, valid if setup tag == 0
    h += b'\x00'             # bmRequestType
    h += b'\x00'             # bRequest
    h += b'\x00\x00'         # wValue
    h += b'\x00\x00'         # wIndex
    h += b'\x00\x00'         # wLength
    #
    h += b'\x00\x00\x00\x00' # ISO/interrupt interval
    h += b'\x00\x00\x00\x00' # ISO start frame
    h += b'\x00\x00\x00\x00' # URB flags
    h += b'\x00\x00\x00\x00' # Number of ISO descriptors

    def __init__(self, req, ts, is_submit):
        self.header = bytearray(pcap_usb_pkt.h)
        self.data = b''
        self.set_urbid(req['id'])
        self.set_urbtype('S' if is_submit else 'C')
        self.set_timestamp(ts)
        self.set_addr_ep(req['addr'], req['ep'])
        if req['type'] in ('SETUP IN', 'SETUP OUT'):
            self.set_transfertype(2) # Control
            self.set_setup(req['setup_data'])
        if req['type'] in ('BULK IN'):
            self.set_addr_ep(req['addr'], 0x80 | req['ep'])
        self.set_data(req['data'])

    def set_urbid(self, urbid):
        self.header[4:8] = struct.pack('>I', urbid)

    def set_urbtype(self, urbtype):
        self.header[8] = ord(urbtype)

    def set_transfertype(self, transfertype):
        self.header[9] = transfertype

    def set_addr_ep(self, addr, ep):
        self.header[11] = addr
        self.header[10] = ep

    def set_timestamp(self, ts):
        self.timestamp = ts
        self.header[20:24] = struct.pack('>I', ts[0]) # seconds
        self.header[24:28] = struct.pack('>I', ts[1]) # microseconds

    def set_data(self, data):
        self.data = data
        self.header[15] = 0
        self.header[36:40] = struct.pack('>I', len(data))

    def set_setup(self, data):
        self.header[14] = 0
        self.header[40:48] = data

    def packet(self):
        return bytes(self.header) + bytes(self.data)

    def record_header(self):
        # See https://wiki.wireshark.org/Development/LibpcapFileFormat.
        (secs, usecs) = self.timestamp
        h  = struct.pack('>I', secs) # TS seconds
        h += struct.pack('>I', usecs) # TS microseconds
        # No truncation, so both lengths are the same.
        h += struct.pack('>I', len(self)) # Captured len (usb hdr + data)
        h += struct.pack('>I', len(self)) # Original len
        return h

    def __len__(self):
        return 64 + len(self.data)

class Decoder(srd.Decoder):
    api_version = 3
    id = 'usb_request'
    name = 'USB request'
    longname = 'Universal Serial Bus (LS/FS) transaction/request'
    desc = 'USB (low-speed/full-speed) transaction/request protocol.'
    license = 'gplv2+'
    inputs = ['usb_packet']
    outputs = ['usb_request']
    options = (
        {'id': 'in_request_start', 'desc': 'Start IN requests on',
            'default': 'submit', 'values': ('submit', 'first-ack')},
    )
    tags = ['PC']
    annotations = (
        ('request-setup-read', 'Setup: Device-to-host'),
        ('request-setup-write', 'Setup: Host-to-device'),
        ('request-bulk-read', 'Bulk: Device-to-host'),
        ('request-bulk-write', 'Bulk: Host-to-device'),
        ('errors', 'Unexpected packets'),
    )
    annotation_rows = (
        ('request-setup', 'USB SETUP', (0, 1)),
        ('request-in', 'USB BULK IN', (2,)),
        ('request-out', 'USB BULK OUT', (3,)),
        ('errors', 'Errors', (4,)),
    )
    binary = (
        ('pcap', 'PCAP format'),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.samplerate = None
        self.request = {}
        self.request_id = 0
        self.transaction_state = 'IDLE'
        self.ss_transaction = None
        self.es_transaction = None
        self.transaction_ep = None
        self.transaction_addr = None
        self.wrote_pcap_header = False

    def putr(self, ss, es, data):
        self.put(ss, es, self.out_ann, data)

    def putb(self, ts, data):
        self.put(ts, ts, self.out_binary, data)

    def pcap_global_header(self):
        # See https://wiki.wireshark.org/Development/LibpcapFileFormat.
        h  = b'\xa1\xb2\xc3\xd4' # Magic, indicate microsecond ts resolution
        h += b'\x00\x02'         # Major version 2
        h += b'\x00\x04'         # Minor version 4
        h += b'\x00\x00\x00\x00' # Correction vs. UTC, seconds
        h += b'\x00\x00\x00\x00' # Timestamp accuracy
        h += b'\xff\xff\xff\xff' # Max packet len
        # LINKTYPE_USB_LINUX_MMAPPED 220
        # Linux usbmon format, see Documentation/usb/usbmon.txt.
        h += b'\x00\x00\x00\xdc' # Link layer
        return h

    def metadata(self, key, value):
        if key == srd.SRD_CONF_SAMPLERATE:
            self.samplerate = value
            if self.samplerate:
                self.secs_per_sample = float(1) / float(self.samplerate)

    def start(self):
        self.out_binary = self.register(srd.OUTPUT_BINARY)
        self.out_ann = self.register(srd.OUTPUT_ANN)
        self.in_request_start = self.options['in_request_start']

    def handle_transfer(self):
        request_started = 0
        request_end = self.handshake in ('ACK', 'STALL', 'timeout')
        ep = self.transaction_ep
        addr = self.transaction_addr

        # Handle protocol STALLs, condition lasts until next SETUP transfer (8.5.3.4)
        if self.transaction_type == 'SETUP' and (addr, ep) in self.request:
            request = self.request[(addr,ep)]
            if request['type'] in ('SETUP IN', 'SETUP OUT'):
                request['es'] = self.ss_transaction
                self.handle_request(0, 1)

        if not (addr, ep) in self.request:
            self.request[(addr, ep)] = {'setup_data': [], 'data': [],
                'type': None, 'ss': self.ss_transaction, 'es': None,
                'ss_data': None, 'id': self.request_id, 'addr': addr, 'ep': ep}
            self.request_id += 1
            request_started = 1
        request = self.request[(addr,ep)]

        if request_end:
            request['es'] = self.es_transaction
            request['handshake'] = self.handshake

        # BULK or INTERRUPT transfer
        if request['type'] in (None, 'BULK IN') and self.transaction_type == 'IN':
            request['type'] = 'BULK IN'
            if len(request['data']) == 0 and len(self.transaction_data) > 0:
                request['ss_data'] = self.ss_transaction
            request['data'] += self.transaction_data
            self.handle_request(request_started, request_end)
        elif request['type'] in (None, 'BULK OUT') and self.transaction_type == 'OUT':
            request['type'] = 'BULK OUT'
            if self.handshake == 'ACK':
                request['data'] += self.transaction_data
            self.handle_request(request_started, request_end)

        # CONTROL, SETUP stage
        elif request['type'] is None and self.transaction_type == 'SETUP':
            request['setup_data'] = self.transaction_data
            request['wLength'] = struct.unpack('<H',
                bytes(self.transaction_data[6:8]))[0]
            if self.transaction_data[0] & 0x80:
                request['type'] = 'SETUP IN'
                self.handle_request(1, 0)
            else:
                request['type'] = 'SETUP OUT'
                self.handle_request(request['wLength'] == 0, 0)

        # CONTROL, DATA stage
        elif request['type'] == 'SETUP IN' and self.transaction_type == 'IN':
            request['data'] += self.transaction_data

        elif request['type'] == 'SETUP OUT' and self.transaction_type == 'OUT':
            if self.handshake == 'ACK':
                request['data'] += self.transaction_data
            if request['wLength'] == len(request['data']):
                self.handle_request(1, 0)

        # CONTROL, STATUS stage
        elif request['type'] == 'SETUP IN' and self.transaction_type == 'OUT':
            self.handle_request(0, request_end)

        elif request['type'] == 'SETUP OUT' and self.transaction_type == 'IN':
            self.handle_request(0, request_end)

        else:
            return

        return

    def ts_from_samplenum(self, sample):
        ts = float(sample) * self.secs_per_sample
        return (int(ts), int((ts % 1.0) * 1e6))

    def write_pcap_header(self):
        if not self.wrote_pcap_header:
            self.put(0, 0, self.out_binary, [0, self.pcap_global_header()])
            self.wrote_pcap_header = True

    def request_summary(self, request):
        s = '['
        if request['type'] in ('SETUP IN', 'SETUP OUT'):
            for b in request['setup_data']:
                s += ' %02X' % b
            s += ' ]['
        for b in request['data']:
            s += ' %02X' % b
        s += ' ] : %s' % request['handshake']
        return s

    def handle_request(self, request_start, request_end):
        if request_start != 1 and request_end != 1:
            return
        self.write_pcap_header()
        ep = self.transaction_ep
        addr = self.transaction_addr
        request = self.request[(addr, ep)]

        ss, es, ss_data = request['ss'], request['es'], request['ss_data']
        if self.in_request_start == 'submit':
            ss_data = ss

        if request_start == 1:
            # Issue PCAP 'SUBMIT' packet.
            ts = self.ts_from_samplenum(ss)
            pkt = pcap_usb_pkt(request, ts, True)
            self.putb(ss, [0, pkt.record_header()])
            self.putb(ss, [0, pkt.packet()])

        if request_end == 1:
            # Write annotation.
            summary = self.request_summary(request)
            if request['type'] == 'SETUP IN':
                self.putr(ss, es, [0, ['SETUP in: %s' % summary]])
            elif request['type'] == 'SETUP OUT':
                self.putr(ss, es, [1, ['SETUP out: %s' % summary]])
            elif request['type'] == 'BULK IN':
                self.putr(ss_data, es, [2, ['BULK in: %s' % summary]])
            elif request['type'] == 'BULK OUT':
                self.putr(ss, es, [3, ['BULK out: %s' % summary]])

            # Issue PCAP 'COMPLETE' packet.
            ts = self.ts_from_samplenum(es)
            pkt = pcap_usb_pkt(request, ts, False)
            self.putb(ss, [0, pkt.record_header()])
            self.putb(ss, [0, pkt.packet()])
            del self.request[(addr, ep)]

    def decode(self, ss, es, data):
        if not self.samplerate:
            raise SamplerateError('Cannot decode without samplerate.')
        ptype, pdata = data

        # We only care about certain packet types for now.
        if ptype not in ('PACKET'):
            return

        pcategory, pname, pinfo = pdata

        if pcategory == 'TOKEN':
            if pname == 'SOF':
                return
            if self.transaction_state == 'TOKEN RECEIVED':
                transaction_timeout = self.es_transaction
                # Token length is 35 bits, timeout is 16..18 bit times
                # (USB 2.0 7.1.19.1).
                transaction_timeout += int((self.es_transaction - self.ss_transaction) / 2)
                if ss > transaction_timeout:
                    self.es_transaction = transaction_timeout
                    self.handshake = 'timeout'
                    self.handle_transfer()
                    self.transaction_state = 'IDLE'

            if self.transaction_state != 'IDLE':
                self.putr(ss, es, [4, ['ERR: received %s token in state %s' %
                    (pname, self.transaction_state)]])
                return

            sync, pid, addr, ep, crc5 = pinfo
            self.transaction_data = []
            self.ss_transaction = ss
            self.es_transaction = es
            self.transaction_state = 'TOKEN RECEIVED'
            self.transaction_ep = ep
            if ep > 0 and pname == 'IN':
                self.transaction_ep = ep + 0x80
            self.transaction_addr = addr
            self.transaction_type = pname # IN OUT SETUP

        elif pcategory == 'DATA':
            if self.transaction_state != 'TOKEN RECEIVED':
                self.putr(ss, es, [4, ['ERR: received %s token in state %s' %
                    (pname, self.transaction_state)]])
                return

            self.transaction_data = pinfo[2]
            self.transaction_state = 'DATA RECEIVED'

        elif pcategory == 'HANDSHAKE':
            if self.transaction_state not in ('TOKEN RECEIVED', 'DATA RECEIVED'):
                self.putr(ss, es, [4, ['ERR: received %s token in state %s' %
                    (pname, self.transaction_state)]])
                return

            self.handshake = pname
            self.transaction_state = 'IDLE'
            self.es_transaction = es
            self.handle_transfer()

        elif pname == 'PRE':
            return

        else:
            self.putr(ss, es, [4, ['ERR: received unhandled %s token in state %s' %
                (pname, self.transaction_state)]])
            return



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2016 Benjamin Larsson <benjamin@southpole.se>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd

class SamplerateError(Exception):
    pass

class Decoder(srd.Decoder):
    api_version = 3
    id = 'em4305'
    name = 'EM4305'
    longname = 'RFID EM4205/EM4305'
    desc = 'EM4205/EM4305 100-150kHz RFID protocol.'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = []
    tags = ['IC', 'RFID']
    channels = (
        {'id': 'data', 'name': 'Data', 'desc': 'Data line'},
    )
    options = (
        {'id': 'coilfreq', 'desc': 'Coil frequency', 'default': 125000},
        {'id': 'first_field_stop', 'desc': 'First field stop min', 'default': 40},
        {'id': 'w_gap', 'desc': 'Write gap min', 'default': 12},
        {'id': 'w_one_max', 'desc': 'Write one max', 'default': 32},
        {'id': 'w_zero_on_min', 'desc': 'Write zero on min', 'default': 15},
        {'id': 'w_zero_off_max', 'desc': 'Write zero off max', 'default': 27},
        {'id': 'em4100_decode', 'desc': 'EM4100 decode', 'default': 'on',
            'values': ('on', 'off')},
    )
    annotations = (
        ('bit_value', 'Bit value'),
        ('first_field_stop', 'First field stop'),
        ('write_gap', 'Write gap'),
        ('write_mode_exit', 'Write mode exit'),
        ('bit', 'Bit'),
        ('opcode', 'Opcode'),
        ('lock', 'Lock'),
        ('data', 'Data'),
        ('password', 'Password'),
        ('address', 'Address'),
        ('bitrate', 'Bitrate'),
    )
    annotation_rows = (
        ('bits', 'Bits', (0,)),
        ('structure', 'Structure', (1, 2, 3, 4)),
        ('fields', 'Fields', (5, 6, 7, 8, 9)),
        ('decode', 'Decode', (10,)),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.samplerate = None
        self.last_samplenum = None
        self.state = 'FFS_SEARCH'
        self.bits_pos = [[0 for col in range(3)] for row in range(70)]
        self.br_string = ['RF/8', 'RF/16', 'Unused', 'RF/32', 'RF/40',
                          'Unused', 'Unused', 'RF/64',]
        self.encoder = ['not used', 'Manchester', 'Bi-phase', 'not used']
        self.delayed_on = ['No delay', 'Delayed on - BP/8', 'Delayed on - BP/4', 'No delay']
        self.em4100_decode1_partial = 0
        self.cmds = ['Invalid', 'Login', 'Write word', 'Invalid', 'Read word', 'Disable', 'Protect', 'Invalid']

    def metadata(self, key, value):
        if key == srd.SRD_CONF_SAMPLERATE:
            self.samplerate = value
        self.field_clock = self.samplerate / self.options['coilfreq']
        self.wzmax = self.options['w_zero_off_max'] * self.field_clock
        self.wzmin = self.options['w_zero_on_min'] * self.field_clock
        self.womax = self.options['w_one_max'] * self.field_clock
        self.ffs = self.options['first_field_stop'] * self.field_clock
        self.writegap = self.options['w_gap'] * self.field_clock
        self.nogap = 300 * self.field_clock

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def decode_config(self, idx):
        bitrate = self.get_3_bits(idx+2)
        self.put(self.bits_pos[idx][1], self.bits_pos[idx+5][2],
                 self.out_ann, [10, ['Data rate: ' + \
                 self.br_string[bitrate], self.br_string[bitrate]]])
        encoding = self.bits_pos[idx+6][0]<<0 | self.bits_pos[idx+7][0]<<1
        self.put(self.bits_pos[idx+6][1], self.bits_pos[idx+10][2],
                 self.out_ann, [10, ['Encoder: ' + \
                 self.encoder[encoding], self.encoder[encoding]]])
        self.put(self.bits_pos[idx+11][1], self.bits_pos[idx+12][2], self.out_ann,
                 [10, ['Zero bits', 'ZB']])
        delay_on = self.bits_pos[idx+13][0]<<0 | self.bits_pos[idx+14][0]<<1
        self.put(self.bits_pos[idx+13][1], self.bits_pos[idx+14][2],
                 self.out_ann, [10, ['Delayed on: ' + \
                 self.delayed_on[delay_on], self.delayed_on[delay_on]]])
        lwr = self.bits_pos[idx+15][0]<<3 | self.bits_pos[idx+16][0]<<2 | \
                 self.bits_pos[idx+18][0]<<1 | self.bits_pos[idx+19][0]<<0
        self.put(self.bits_pos[idx+15][1], self.bits_pos[idx+19][2],
                 self.out_ann, [10, ['Last default read word: %d' % lwr, 'LWR: %d' % lwr, '%d' % lwr]])
        self.put(self.bits_pos[idx+20][1], self.bits_pos[idx+20][2],
                 self.out_ann, [10, ['Read login: %d' % self.bits_pos[idx+20][0], '%d' % self.bits_pos[idx+20][0]]])
        self.put(self.bits_pos[idx+21][1], self.bits_pos[idx+21][2], self.out_ann,
                 [10, ['Zero bits', 'ZB']])
        self.put(self.bits_pos[idx+22][1], self.bits_pos[idx+22][2],
                 self.out_ann, [10, ['Write login: %d' % self.bits_pos[idx+22][0], '%d' % self.bits_pos[idx+22][0]]])
        self.put(self.bits_pos[idx+23][1], self.bits_pos[idx+24][2], self.out_ann,
                 [10, ['Zero bits', 'ZB']])
        self.put(self.bits_pos[idx+25][1], self.bits_pos[idx+25][2],
                 self.out_ann, [10, ['Disable: %d' % self.bits_pos[idx+25][0], '%d' % self.bits_pos[idx+25][0]]])
        self.put(self.bits_pos[idx+27][1], self.bits_pos[idx+27][2],
                 self.out_ann, [10, ['Reader talk first: %d' % self.bits_pos[idx+27][0], 'RTF: %d' % self.bits_pos[idx+27][0]]])
        self.put(self.bits_pos[idx+28][1], self.bits_pos[idx+28][2], self.out_ann,
                 [10, ['Zero bits', 'ZB']])
        self.put(self.bits_pos[idx+29][1], self.bits_pos[idx+29][2],
                 self.out_ann, [10, ['Pigeon mode: %d' % self.bits_pos[idx+29][0], '%d' % self.bits_pos[idx+29][0]]])
        self.put(self.bits_pos[idx+30][1], self.bits_pos[idx+34][2],
                 self.out_ann, [10, ['Reserved', 'Res', 'R']])

    def put4bits(self, idx):
        bits = self.bits_pos[idx][0]<<3 | self.bits_pos[idx+1][0]<<2 | \
               self.bits_pos[idx+2][0]<<1 | self.bits_pos[idx+3][0]
        self.put(self.bits_pos[idx][1], self.bits_pos[idx+3][2], self.out_ann,
                 [10, ['%X' % bits]])

    def em4100_decode1(self, idx):
        self.put(self.bits_pos[idx][1], self.bits_pos[idx+9][2], self.out_ann,
                 [10, ['EM4100 header', 'EM header', 'Header', 'H']])
        self.put4bits(idx+10)
        bits = self.bits_pos[idx+15][0]<<3 | self.bits_pos[idx+16][0]<<2 | \
               self.bits_pos[idx+18][0]<<1 | self.bits_pos[idx+19][0]<<0
        self.put(self.bits_pos[idx+15][1], self.bits_pos[idx+19][2], self.out_ann,
               [10, ['%X' % bits]])
        self.put4bits(idx+21)
        self.put4bits(idx+27)
        self.em4100_decode1_partial = self.bits_pos[idx+32][0]<<3 | \
            self.bits_pos[idx+33][0]<<2 | self.bits_pos[idx+34][0]<<1
        self.put(self.bits_pos[idx+32][1], self.bits_pos[idx+34][2],
                 self.out_ann, [10, ['Partial nibble']])

    def em4100_decode2(self, idx):
        if self.em4100_decode1_partial != 0:
            bits = self.em4100_decode1_partial + self.bits_pos[idx][0]
            self.put(self.bits_pos[idx][1], self.bits_pos[idx][2],
                     self.out_ann, [10, ['%X' % bits]])
            self.em4100_decode1_partial = 0
        else:
            self.put(self.bits_pos[idx][1], self.bits_pos[idx][2],
                     self.out_ann, [10, ['Partial nibble']])

        self.put4bits(idx+2)
        bits = self.bits_pos[idx+7][0]<<3 | self.bits_pos[idx+9][0]<<2 | \
               self.bits_pos[idx+10][0]<<1 | self.bits_pos[idx+11][0]<<0
        self.put(self.bits_pos[idx+7][1], self.bits_pos[idx+11][2], self.out_ann,
               [10, ['%X' % bits]])
        self.put4bits(idx+13)
        self.put4bits(idx+19)
        bits = self.bits_pos[idx+24][0]<<3 | self.bits_pos[idx+25][0]<<2 | \
               self.bits_pos[idx+27][0]<<1 | self.bits_pos[idx+28][0]<<0
        self.put(self.bits_pos[idx+24][1], self.bits_pos[idx+28][2], self.out_ann,
               [10, ['%X' % bits]])
        self.put(self.bits_pos[idx+30][1], self.bits_pos[idx+34][2],
                 self.out_ann, [10, ['EM4100 trailer']])

    def get_32_bits(self, idx):
        return self.get_8_bits(idx+27)<<24 | self.get_8_bits(idx+18)<<16 | \
               self.get_8_bits(idx+9)<<8 | self.get_8_bits(idx)

    def get_8_bits(self, idx):
        retval = 0
        for i in range(0, 8):
            retval <<= 1
            retval |= self.bits_pos[i+idx][0]
        return retval

    def get_3_bits(self, idx):
        return self.bits_pos[idx][0]<<2 | self.bits_pos[idx+1][0]<<1 | \
               self.bits_pos[idx+2][0]

    def get_4_bits(self, idx):
        return self.bits_pos[idx][0]<<0 | self.bits_pos[idx+1][0]<<1 | \
               self.bits_pos[idx+2][0]<<2 | self.bits_pos[idx+3][0]<<3

    def print_row_parity(self, idx, length):
        parity = 0
        for i in range(0, length):
            parity += self.bits_pos[i+idx][0]
        parity = parity & 0x1
        if parity == self.bits_pos[idx+length][0]:
            self.put(self.bits_pos[idx+length][1], self.bits_pos[idx+length][2], self.out_ann,
                [5, ['Row parity OK', 'Parity OK', 'OK']])
        else:
            self.put(self.bits_pos[idx+length][1], self.bits_pos[idx+length][2], self.out_ann,
                [5, ['Row parity failed', 'Parity failed', 'Fail']])

    def print_col_parity(self, idx):
        data_1 = self.get_8_bits(idx)
        data_2 = self.get_8_bits(idx+9)
        data_3 = self.get_8_bits(idx+9+9)
        data_4 = self.get_8_bits(idx+9+9+9)
        col_par = self.get_8_bits(idx+9+9+9+9)
        col_par_calc = data_1^data_2^data_3^data_4

        if col_par == col_par_calc:
            self.put(self.bits_pos[idx+9+9+9+9][1], self.bits_pos[idx+9+9+9+9+7][2], self.out_ann,
                [5, ['Column parity OK', 'Parity OK', 'OK']])
        else:
            self.put(self.bits_pos[idx+9+9+9+9][1], self.bits_pos[idx+9+9+9+9+7][2], self.out_ann,
                [5, ['Column parity failed', 'Parity failed', 'Fail']])

    def print_8bit_data(self, idx):
        data = self.get_8_bits(idx)
        self.put(self.bits_pos[idx][1], self.bits_pos[idx+7][2], self.out_ann,
                     [9, ['Data' + ': %X' % data, '%X' % data]])

    def put_fields(self):
        if self.bit_nr == 50:
            self.put(self.bits_pos[0][1], self.bits_pos[0][2], self.out_ann,
                     [4, ['Logic zero']])
            self.put(self.bits_pos[1][1], self.bits_pos[4][2], self.out_ann,
                     [4, ['Command', 'Cmd', 'C']])
            self.put(self.bits_pos[5][1], self.bits_pos[49][2], self.out_ann,
                     [4, ['Password', 'Passwd', 'Pass', 'P']])
            # Get command.
            cmd = self.get_3_bits(1)
            self.put(self.bits_pos[1][1], self.bits_pos[3][2], self.out_ann,
                     [5, [self.cmds[cmd]]])
            self.print_row_parity(1, 3)

            # Print data.
            self.print_8bit_data(5)
            self.print_row_parity(5, 8)
            self.print_8bit_data(14)
            self.print_row_parity(14, 8)
            self.print_8bit_data(23)
            self.print_row_parity(23, 8)
            self.print_8bit_data(32)
            self.print_row_parity(32, 8)
            self.print_col_parity(5)
            if self.bits_pos[49][0] == 0:
                self.put(self.bits_pos[49][1], self.bits_pos[49][2], self.out_ann,
                         [5, ['Stop bit', 'Stop', 'SB']])
            else:
                self.put(self.bits_pos[49][1], self.bits_pos[49][2], self.out_ann,
                         [5, ['Stop bit error', 'Error']])

            if cmd == 1:
                password = self.get_32_bits(5)
                self.put(self.bits_pos[12][1], self.bits_pos[46][2], self.out_ann,
                     [10, ['Login password: %X' % password]])

        if self.bit_nr == 57:
            self.put(self.bits_pos[0][1], self.bits_pos[0][2], self.out_ann,
                     [4, ['Logic zero', 'LZ']])
            self.put(self.bits_pos[1][1], self.bits_pos[4][2], self.out_ann,
                     [4, ['Command', 'Cmd', 'C']])
            self.put(self.bits_pos[5][1], self.bits_pos[11][2], self.out_ann,
                     [4, ['Address', 'Addr', 'A']])
            self.put(self.bits_pos[12][1], self.bits_pos[56][2], self.out_ann,
                     [4, ['Data', 'Da', 'D']])

            # Get command.
            cmd = self.get_3_bits(1)
            self.put(self.bits_pos[1][1], self.bits_pos[3][2], self.out_ann,
                     [5, [self.cmds[cmd]]])
            self.print_row_parity(1, 3)

            # Get address.
            addr = self.get_4_bits(5)
            self.put(self.bits_pos[5][1], self.bits_pos[8][2], self.out_ann,
                     [9, ['Addr' + ': %d' % addr, '%d' % addr]])
            self.put(self.bits_pos[9][1], self.bits_pos[10][2], self.out_ann,
                     [5, ['Zero bits', 'ZB']])
            self.print_row_parity(5, 6)
            # Print data.
            self.print_8bit_data(12)
            self.print_row_parity(12, 8)
            self.print_8bit_data(21)
            self.print_row_parity(21, 8)
            self.print_8bit_data(30)
            self.print_row_parity(30, 8)
            self.print_8bit_data(39)
            self.print_row_parity(39, 8)
            self.print_col_parity(12)
            if self.bits_pos[56][0] == 0:
                self.put(self.bits_pos[56][1], self.bits_pos[56][2], self.out_ann,
                         [5, ['Stop bit', 'Stop', 'SB']])
            else:
                self.put(self.bits_pos[56][1], self.bits_pos[56][2], self.out_ann,
                         [5, ['Stop bit error', 'Error']])

            if addr == 4:
                self.decode_config(12)

            if addr == 2:
                password = self.get_32_bits(12)
                self.put(self.bits_pos[12][1], self.bits_pos[46][2], self.out_ann,
                     [10, ['Write password: %X' % password]])

            # If we are programming EM4100 data we can decode it halfway.
            if addr == 5 and self.options['em4100_decode'] == 'on':
                self.em4100_decode1(12)
            if addr == 6 and self.options['em4100_decode'] == 'on':
                self.em4100_decode2(12)

        self.bit_nr = 0

    def add_bits_pos(self, bit, ss_bit, es_bit):
        if self.bit_nr < 70:
            self.bits_pos[self.bit_nr][0] = bit
            self.bits_pos[self.bit_nr][1] = ss_bit
            self.bits_pos[self.bit_nr][2] = es_bit
            self.bit_nr += 1

    def decode(self):
        if not self.samplerate:
            raise SamplerateError('Cannot decode without samplerate.')

        # Initialize internal state.
        self.last_samplenum = self.samplenum
        self.oldsamplenum = 0
        self.old_gap_end = 0
        self.gap_detected = 0
        self.bit_nr = 0

        while True:
            # Ignore identical samples, only process edges.
            (pin,) = self.wait({0: 'e'})

            pl = self.samplenum - self.oldsamplenum
            pp = pin
            samples = self.samplenum - self.last_samplenum

            if self.state == 'FFS_DETECTED':
                if pl > self.writegap:
                    self.gap_detected = 1
                if (self.last_samplenum - self.old_gap_end) > self.nogap:
                    self.gap_detected = 0
                    self.state = 'FFS_SEARCH'
                    self.put(self.old_gap_end, self.last_samplenum,
                             self.out_ann, [3, ['Write mode exit']])
                    self.put_fields()

            if self.state == 'FFS_SEARCH':
                if pl > self.ffs:
                    self.gap_detected = 1
                    self.put(self.last_samplenum, self.samplenum,
                             self.out_ann, [1, ['First field stop', 'Field stop', 'FFS']])
                    self.state = 'FFS_DETECTED'

            if self.gap_detected == 1:
                self.gap_detected = 0
                if (self.last_samplenum - self.old_gap_end) > self.wzmin \
                        and (self.last_samplenum - self.old_gap_end) < self.wzmax:
                    self.put(self.old_gap_end, self.samplenum,
                             self.out_ann, [0, ['0']])
                    self.add_bits_pos(0, self.old_gap_end, self.samplenum)
                if (self.last_samplenum - self.old_gap_end) > self.womax \
                        and (self.last_samplenum-self.old_gap_end) < self.nogap:
                    # One or more 1 bits
                    one_bits = (int)((self.last_samplenum - self.old_gap_end) / self.womax)
                    for ox in range(0, one_bits):
                        bs = (int)(self.old_gap_end+ox*self.womax)
                        be = (int)(self.old_gap_end+ox*self.womax + self.womax)
                        self.put(bs, be, self.out_ann, [0, ['1']])
                        self.add_bits_pos(1, bs, be)
                    if (self.samplenum - self.last_samplenum) > self.wzmin \
                            and (self.samplenum - self.last_samplenum) < self.wzmax:
                        bs = (int)(self.old_gap_end+one_bits*self.womax)
                        self.put(bs, self.samplenum, self.out_ann, [0, ['0']])
                        self.add_bits_pos(0, bs, self.samplenum)

                self.old_gap_end = self.samplenum

            if self.state == 'SKIP':
                self.state = 'FFS_SEARCH'

            self.oldsamplenum = self.samplenum
            self.last_samplenum = self.samplenum



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2017 Joel Holdsworth <joel@airwebreathe.org.uk>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd

def disabled_enabled(v):
    return ['Disabled', 'Enabled'][v]

def output_power(v):
    return '%+ddBm' % [-4, -1, 2, 5][v]

regs = {
# reg:   name                        offset width parser
    0: [
        ('FRAC',                          3, 12, None),
        ('INT',                          15, 16, lambda v: 'Not Allowed' if v < 32 else v)
    ],
    1: [
        ('MOD',                           3, 12, None),
        ('Phase',                        15, 12, None),
        ('Prescalar',                    27,  1, lambda v: ['4/5', '8/9'][v]),
        ('Phase Adjust',                 28,  1, lambda v: ['Off', 'On'][v]),
    ],
    2: [
        ('Counter Reset',                 3,  1, disabled_enabled),
        ('Charge Pump Three-State',       4,  1, disabled_enabled),
        ('Power-Down',                    5,  1, disabled_enabled),
        ('PD Polarity',                   6,  1, lambda v: ['Negative', 'Positive'][v]),
        ('LDP',                           7,  1, lambda v: ['10ns', '6ns'][v]),
        ('LDF',                           8,  1, lambda v: ['FRAC-N', 'INT-N'][v]),
        ('Charge Pump Current Setting',   9,  4, lambda v: '%0.2fmA @ 5.1kΩ' %
            [0.31, 0.63, 0.94, 1.25, 1.56, 1.88, 2.19, 2.50,
            2.81, 3.13, 3.44, 3.75, 4.06, 4.38, 4.69, 5.00][v]),
        ('Double Buffer',                13,  1, disabled_enabled),
        ('R Counter',                    14, 10, None),
        ('RDIV2',                        24,  1, disabled_enabled),
        ('Reference Doubler',            25,  1, disabled_enabled),
        ('MUXOUT',                       26,  3, lambda v:
            ['Three-State Output', 'DVdd', 'DGND', 'R Counter Output', 'N Divider Output',
            'Analog Lock Detect', 'Digital Lock Detect', 'Reserved'][v]),
        ('Low Noise and Low Spur Modes', 29,  2, lambda v:
            ['Low Noise Mode', 'Reserved', 'Reserved', 'Low Spur Mode'][v])
    ],
    3: [
        ('Clock Divider',                 3, 12, None),
        ('Clock Divider Mode',           15,  2, lambda v:
            ['Clock Divider Off', 'Fast Lock Enable', 'Resync Enable', 'Reserved'][v]),
        ('CSR Enable',                   18,  1, disabled_enabled),
        ('Charge Cancellation',          21,  1, disabled_enabled),
        ('ABP',                          22,  1, lambda v: ['6ns (FRAC-N)', '3ns (INT-N)'][v]),
        ('Band Select Clock Mode',       23,  1, lambda v: ['Low', 'High'][v])
    ],
    4: [
        ('Output Power',                  3,  2, output_power),
        ('Output Enable',                 5,  1, disabled_enabled),
        ('AUX Output Power',              6,  2, output_power),
        ('AUX Output Select',             8,  1, lambda v: ['Divided Output', 'Fundamental'][v]),
        ('AUX Output Enable',             9,  1, disabled_enabled),
        ('MTLD',                         10,  1, disabled_enabled),
        ('VCO Power-Down',               11,  1, lambda v:
            'VCO Powered ' + ('Down' if v == 1 else 'Up')),
        ('Band Select Clock Divider',    12,  8, None),
        ('RF Divider Select',            20,  3, lambda v: '÷' + str(2**v)),
        ('Feedback Select',              23,  1, lambda v: ['Divided', 'Fundamental'][v]),
    ],
    5: [
        ('LD Pin Mode',                  22,  2, lambda v:
            ['Low', 'Digital Lock Detect', 'Low', 'High'][v])
    ]
}

ANN_REG = 0

class Decoder(srd.Decoder):
    api_version = 3
    id = 'adf435x'
    name = 'ADF435x'
    longname = 'Analog Devices ADF4350/1'
    desc = 'Wideband synthesizer with integrated VCO.'
    license = 'gplv3+'
    inputs = ['spi']
    outputs = []
    tags = ['Clock/timing', 'IC', 'Wireless/RF']
    annotations = (
        # Sent from the host to the chip.
        ('register', 'Register written to the device'),
    )
    annotation_rows = (
        ('registers', 'Register writes', (ANN_REG,)),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.bits = []

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def decode_bits(self, offset, width):
        return (sum([(1 << i) if self.bits[offset + i][0] else 0 for i in range(width)]),
            (self.bits[offset + width - 1][1], self.bits[offset][2]))

    def decode_field(self, name, offset, width, parser):
        val, pos = self.decode_bits(offset, width)
        self.put(pos[0], pos[1], self.out_ann, [ANN_REG,
            ['%s: %s' % (name, parser(val) if parser else str(val))]])
        return val

    def decode(self, ss, es, data):

        ptype, data1, data2 = data

        if ptype == 'CS-CHANGE':
            if data1 == 1:
                if len(self.bits) == 32:
                    reg_value, reg_pos = self.decode_bits(0, 3)
                    self.put(reg_pos[0], reg_pos[1], self.out_ann, [ANN_REG,
                        ['Register: %d' % reg_value, 'Reg: %d' % reg_value,
                         '[%d]' % reg_value]])
                    if reg_value < len(regs):
                        field_descs = regs[reg_value]
                        for field_desc in field_descs:
                            field = self.decode_field(*field_desc)
                self.bits = []
        if ptype == 'BITS':
            self.bits = data1 + self.bits



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2017 Christoph Rackwitz <christoph.rackwitz@rwth-aachen.de>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd

def decode_ditdah(s):
    return tuple({'-': 3, '.': 1}[c] for c in s)

def encode_ditdah(tpl):
    return ''.join({1: '.', 3: '-'}[c] for c in tpl)

# https://www.itu.int/dms_pubrec/itu-r/rec/m/R-REC-M.1677-1-200910-I!!PDF-E.pdf
# Recommendation ITU-R M.1677-1
# (10/2009)
# International Morse code
alphabet = {
    # 1.1.1 Letters
    '.-':       'a',
    '-...':     'b',
    '-.-.':     'c',
    '-..':      'd',
    '.':        'e',
    '..-..':    'é', # "accented"
    '..-.':     'f',
    '--.':      'g',
    '....':     'h',
    '..':       'i',
    '.---':     'j',
    '-.-':      'k',
    '.-..':     'l',
    '--':       'm',
    '-.':       'n',
    '---':      'o',
    '.--.':     'p',
    '--.-':     'q',
    '.-.':      'r',
    '...':      's',
    '-':        't',
    '..-':      'u',
    '...-':     'v',
    '.--':      'w',
    '-..-':     'x',
    '-.--':     'y',
    '--..':     'z',

    # 1.1.2 Figures
    '.----':    '1',
    '..---':    '2',
    '...--':    '3',
    '....-':    '4',
    '.....':    '5',
    '-....':    '6',
    '--...':    '7',
    '---..':    '8',
    '----.':    '9',
    '-----':    '0',

    # 1.1.3 Punctuation marks and miscellaneous signs
    '.-.-.-':   '.',          # Full stop (period)
    '--..--':   ',',          # Comma
    '---...':   ':',          # Colon or division sign
    '..--..':   '?',          # Question mark (note of interrogation or request for repetition of a transmission not understood)
    '.----.':   '’',          # Apostrophe
    '-....-':   '-',          # Hyphen or dash or subtraction sign
    '-..-.':    '/',          # Fraction bar or division sign
    '-.--.':    '(',          # Left-hand bracket (parenthesis)
    '-.--.-':   ')',          # Right-hand bracket (parenthesis)
    '.-..-.':   '“ ”',        # Inverted commas (quotation marks) (before and after the words)
    '-...-':    '=',          # Double hyphen
    '...-.':    'UNDERSTOOD', # Understood
    '........': 'ERROR',      # Error (eight dots)
    '.-.-.':    '+',          # Cross or addition sign
    '.-...':    'WAIT',       # Wait
    '...-.-':   'EOW',        # End of work
    '-.-.-':    'START',      # Starting signal (to precede every transmission)
    '.--.-.':   '@',          # Commercial at

    #'-.-':      'ITT',        # K: Invitation to transmit

    # 3.2.1 For the multiplication sign, the signal corresponding to the letter X shall be transmitted.
    #'-..-':     '×',          # Multiplication sign
}

alphabet = {decode_ditdah(k): v for k, v in alphabet.items()}

# 2 Spacing and length of the signals (right side is just for printing).
symbols = { # level, time units
    # 2.1 A dash is equal to three dots.
    (1, 1): '*',
    (1, 3): '===',
    # 2.2 The space between the signals forming the same letter is equal to one dot.
    (0, 1): '_',
    # 2.3 The space between two letters is equal to three dots.
    (0, 3): '__',
    # 2.4 The space between two words is equal to seven dots.
    (0, 7): '___',
}

class Decoder(srd.Decoder):
    api_version = 3
    id = 'morse'
    name = 'Morse'
    longname = 'Morse code'
    desc = 'Demodulated morse code protocol.'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = []
    tags = ['Encoding']
    channels = (
        {'id': 'data', 'name': 'Data', 'desc': 'Data line'},
    )
    options = (
        {'id': 'timeunit', 'desc': 'Time unit (guess)', 'default': 0.1},
    )
    annotations = (
        ('time', 'Time'),
        ('units', 'Units'),
        ('symbol', 'Symbol'),
        ('letter', 'Letter'),
        ('word', 'Word'),
    )
    annotation_rows = tuple((u, v, (i,)) for i, (u, v) in enumerate(annotations))

    def __init__(self):
        self.reset()

    def reset(self):
        self.samplerate = None

    def metadata(self, key, value):
        if key == srd.SRD_CONF_SAMPLERATE:
            self.samplerate = value

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)
        self.out_binary = self.register(srd.OUTPUT_BINARY)

    def decode_symbols(self):
        # Annotate symbols, emit symbols, handle timeout via token.

        timeunit = self.options['timeunit']

        self.wait({0: 'r'})
        prevtime = self.samplenum # Time of an actual edge.

        while True:
            (val,) = self.wait([{0: 'e'}, {'skip': int(5 * self.samplerate * timeunit)}])

            pval = 1 - val
            curtime = self.samplenum
            dt = (curtime - prevtime) / self.samplerate
            units = dt / timeunit
            iunits = int(max(1, round(units)))
            error = abs(units - iunits)

            symbol = (pval, iunits)

            if self.matched[1]:
                yield None # Flush word.
                continue

            self.put(prevtime, curtime, self.out_ann, [0, ['{:.3g}'.format(dt)]])

            if symbol in symbols:
                self.put(prevtime, curtime, self.out_ann, [1, ['{:.1f}*{:.3g}'.format(units, timeunit)]])
                yield (prevtime, curtime, symbol)
            else:
                self.put(prevtime, curtime, self.out_ann, [1, ['!! {:.1f}*{:.3g} !!'.format(units, timeunit)]])

            prevtime = curtime

            thisunit = dt / iunits
            timeunit += (thisunit - timeunit) * 0.2 * max(0, 1 - 2*error) # Adapt.

    def decode_morse(self):
        # Group symbols into letters.
        sequence = ()
        s0 = s1 = None

        for item in self.decode_symbols():
            do_yield = False
            if item is not None: # Level + width.
                (t0, t1, symbol) = item
                (sval, sunits) = symbol
                if sval == 1:
                    if s0 is None:
                        s0 = t0
                    s1 = t1
                    sequence += (sunits,)
                else:
                    # Generate "flush" for end of letter, end of word.
                    if sunits >= 3:
                        do_yield = True
            else:
                do_yield = True
            if do_yield:
                if sequence:
                    yield (s0, s1, alphabet.get(sequence, encode_ditdah(sequence)))
                    sequence = ()
                    s0 = s1 = None
            if item is None:
                yield None # Pass through flush of 5+ space.

    def decode(self):

        # Strictly speaking there is no point in running this decoder
        # when the sample rate is unknown or zero. But the previous
        # implementation already fell back to a rate of 1 in that case.
        # We stick with this approach, to not introduce new constraints
        # for existing use scenarios.
        if not self.samplerate:
            self.samplerate = 1.0

        # Annotate letters, group into words.
        s0 = s1 = None
        word = ''
        for item in self.decode_morse():
            do_yield = False

            if item is not None: # Append letter.
                (t0, t1, letter) = item
                self.put(t0, t1, self.out_ann, [3, [letter]])
                if s0 is None:
                    s0 = t0
                s1 = t1
                word += letter
            else:
                do_yield = True

            if do_yield: # Flush of word.
                if word:
                    self.put(s0, s1, self.out_ann, [4, [word]])
                    word = ''
                    s0 = s1 = None



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2014 Johannes Roemer <jroemer@physik.uni-wuerzburg.de>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd

# Define valid timing values (in microseconds).
timing = {
    'START LOW'     : {'min': 750, 'max': 25000},
    'START HIGH'    : {'min': 10, 'max': 10000},
    'RESPONSE LOW'  : {'min': 50, 'max': 90},
    'RESPONSE HIGH' : {'min': 50, 'max': 90},
    'BIT LOW'       : {'min': 45, 'max': 90},
    'BIT 0 HIGH'    : {'min': 20, 'max': 35},
    'BIT 1 HIGH'    : {'min': 65, 'max': 80},
}

class SamplerateError(Exception):
    pass

class Decoder(srd.Decoder):
    api_version = 3
    id = 'am230x'
    name = 'AM230x'
    longname = 'Aosong AM230x/DHTxx/RHTxx'
    desc = 'Aosong AM230x/DHTxx/RHTxx humidity/temperature sensor.'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = []
    tags = ['IC', 'Sensor']
    channels = (
        {'id': 'sda', 'name': 'SDA', 'desc': 'Single wire serial data line'},
    )
    options = (
        {'id': 'device', 'desc': 'Device type',
            'default': 'am230x', 'values': ('am230x/rht', 'dht11')},
    )
    annotations = (
        ('start', 'Start'),
        ('response', 'Response'),
        ('bit', 'Bit'),
        ('end', 'End'),
        ('byte', 'Byte'),
        ('humidity', 'Relative humidity in percent'),
        ('temperature', 'Temperature in degrees Celsius'),
        ('checksum', 'Checksum'),
    )
    annotation_rows = (
        ('bits', 'Bits', (0, 1, 2, 3)),
        ('bytes', 'Bytes', (4,)),
        ('results', 'Results', (5, 6, 7)),
    )

    def putfs(self, data):
        self.put(self.fall, self.samplenum, self.out_ann, data)

    def putb(self, data):
        self.put(self.bytepos[-1], self.samplenum, self.out_ann, data)

    def putv(self, data):
        self.put(self.bytepos[-2], self.samplenum, self.out_ann, data)

    def reset_variables(self):
        self.state = 'WAIT FOR START LOW'
        self.fall = 0
        self.rise = 0
        self.bits = []
        self.bytepos = []

    def is_valid(self, name):
        dt = 0
        if name.endswith('LOW'):
            dt = self.samplenum - self.fall
        elif name.endswith('HIGH'):
            dt = self.samplenum - self.rise
        if dt >= self.cnt[name]['min'] and dt <= self.cnt[name]['max']:
            return True
        return False

    def bits2num(self, bitlist):
        number = 0
        for i in range(len(bitlist)):
            number += bitlist[-1 - i] * 2**i
        return number

    def calculate_humidity(self, bitlist):
        h = 0
        if self.options['device'] == 'dht11':
            h = self.bits2num(bitlist[0:8])
        else:
            h = self.bits2num(bitlist) / 10
        return h

    def calculate_temperature(self, bitlist):
        t = 0
        if self.options['device'] == 'dht11':
            t = self.bits2num(bitlist[0:8])
        else:
            t = self.bits2num(bitlist[1:]) / 10
            if bitlist[0] == 1:
                t = -t
        return t

    def calculate_checksum(self, bitlist):
        checksum = 0
        for i in range(8, len(bitlist) + 1, 8):
            checksum += self.bits2num(bitlist[i-8:i])
        return checksum % 256

    def __init__(self):
        self.reset()

    def reset(self):
        self.samplerate = None
        self.reset_variables()

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def metadata(self, key, value):
        if key != srd.SRD_CONF_SAMPLERATE:
            return
        self.samplerate = value
        # Convert microseconds to sample counts.
        self.cnt = {}
        for e in timing:
            self.cnt[e] = {}
            for t in timing[e]:
                self.cnt[e][t] = timing[e][t] * self.samplerate / 1000000

    def handle_byte(self, bit):
        self.bits.append(bit)
        self.putfs([2, ['Bit: %d' % bit, '%d' % bit]])
        self.fall = self.samplenum
        self.state = 'WAIT FOR BIT HIGH'
        if len(self.bits) % 8 == 0:
            byte = self.bits2num(self.bits[-8:])
            self.putb([4, ['Byte: %#04x' % byte, '%#04x' % byte]])
            if len(self.bits) == 16:
                h = self.calculate_humidity(self.bits[-16:])
                self.putv([5, ['Humidity: %.1f %%' % h, 'RH = %.1f %%' % h]])
            elif len(self.bits) == 32:
                t = self.calculate_temperature(self.bits[-16:])
                self.putv([6, ['Temperature: %.1f °C' % t, 'T = %.1f °C' % t]])
            elif len(self.bits) == 40:
                parity = self.bits2num(self.bits[-8:])
                if parity == self.calculate_checksum(self.bits[0:32]):
                    self.putb([7, ['Checksum: OK', 'OK']])
                else:
                    self.putb([7, ['Checksum: not OK', 'NOK']])
                self.state = 'WAIT FOR END'
            self.bytepos.append(self.samplenum)

    def decode(self):
        if not self.samplerate:
            raise SamplerateError('Cannot decode without samplerate.')
        while True:
            # State machine.
            if self.state == 'WAIT FOR START LOW':
                self.wait({0: 'f'})
                self.fall = self.samplenum
                self.state = 'WAIT FOR START HIGH'
            elif self.state == 'WAIT FOR START HIGH':
                self.wait({0: 'r'})
                if self.is_valid('START LOW'):
                    self.rise = self.samplenum
                    self.state = 'WAIT FOR RESPONSE LOW'
                else:
                    self.reset_variables()
            elif self.state == 'WAIT FOR RESPONSE LOW':
                self.wait({0: 'f'})
                if self.is_valid('START HIGH'):
                    self.putfs([0, ['Start', 'S']])
                    self.fall = self.samplenum
                    self.state = 'WAIT FOR RESPONSE HIGH'
                else:
                    self.reset_variables()
            elif self.state == 'WAIT FOR RESPONSE HIGH':
                self.wait({0: 'r'})
                if self.is_valid('RESPONSE LOW'):
                    self.rise = self.samplenum
                    self.state = 'WAIT FOR FIRST BIT'
                else:
                    self.reset_variables()
            elif self.state == 'WAIT FOR FIRST BIT':
                self.wait({0: 'f'})
                if self.is_valid('RESPONSE HIGH'):
                    self.putfs([1, ['Response', 'R']])
                    self.fall = self.samplenum
                    self.bytepos.append(self.samplenum)
                    self.state = 'WAIT FOR BIT HIGH'
                else:
                    self.reset_variables()
            elif self.state == 'WAIT FOR BIT HIGH':
                self.wait({0: 'r'})
                if self.is_valid('BIT LOW'):
                    self.rise = self.samplenum
                    self.state = 'WAIT FOR BIT LOW'
                else:
                    self.reset_variables()
            elif self.state == 'WAIT FOR BIT LOW':
                self.wait({0: 'f'})
                if self.is_valid('BIT 0 HIGH'):
                    bit = 0
                elif self.is_valid('BIT 1 HIGH'):
                    bit = 1
                else:
                    self.reset_variables()
                    continue
                self.handle_byte(bit)
            elif self.state == 'WAIT FOR END':
                self.wait({0: 'r'})
                self.putfs([3, ['End', 'E']])
                self.reset_variables()
